from unittest import TestCase
from autosubmit.config.basicConfig import BasicConfig


class TestBasicConfig(TestCase):

    def test__update_config(self):
        BasicConfig._update_config()

    def test_read(self):
        BasicConfig.read()



from unittest import TestCase
from autosubmit.config.config_common import AutosubmitConfig, BasicConfig
import datetime


class TestAutosubmitConfig(TestCase):

    def setUp(self):
        self.config = AutosubmitConfig('a000')
        self.config.reload()

    def getFullPath(self, relativepath):
        return '/home/Earth/jvegas/debug/autosubmit/a000/' + relativepath

    def test_experiment_file(self):
        self.assertEqual(self.config.experiment_file, self.getFullPath('conf/expdef_a000.conf'))

    def test_platforms_file(self):
        self.assertEqual(self.config.platforms_file, self.getFullPath('conf/platforms_a000.conf'))

    def test_project_file(self):
        self.assertEqual(self.config.project_file, self.getFullPath('conf/proj_a000.conf'))

    def test_jobs_file(self):
        self.assertEqual(self.config.jobs_file, self.getFullPath('conf/jobs_a000.conf'))

    def test_get_project_dir(self):
        self.assertEqual(self.config.get_project_dir(), self.getFullPath(BasicConfig.LOCAL_PROJ_DIR + '/'))

    def test_get_wallclock(self):
        self.assertEqual(self.config.get_wallclock('INI'), '00:05')

    def test_get_processors(self):
        self.assertEqual(self.config.get_processors('INI'), 1)

    def test_get_threads(self):
        self.assertEqual(self.config.get_threads('INI'), 1)

    def test_get_tasks(self):
        self.assertEqual(self.config.get_tasks('INI'), 1)

    def test_check_conf_files(self):
        self.assertTrue(self.config.check_conf_files())

    def test__check_autosubmit_conf(self):
        self.assertTrue(self.config._check_autosubmit_conf())

    def test__check_platforms_conf(self):
        self.assertTrue(self.config._check_platforms_conf())

    def test__check_jobs_conf(self):
        self.assertTrue(self.config._check_jobs_conf())

    def test__check_expdef_conf(self):
        self.assertTrue(self.config._check_expdef_conf())

    def test_check_proj(self):
        self.assertTrue(self.config.check_proj())

    def test_reload(self):
        self.config.reload()

    def test_load_parameters(self):
        self.assertDictEqual(self.config.load_parameters(),
                             {'EXPID': 'a000', 'TOTALJOBS': '6', 'PROJECT_REVISION': '', 'RETRIALS': '4',
                              'HPCARCH': 'patata', 'AUTOSUBMIT_VERSION': '3.0.0rc1', 'CALENDAR': 'standard',
                              'CHUNKLIST': '', 'CHUNKSIZE': '4', 'CHUNKSIZEUNIT': 'month', 'DATELIST': '1956',
                              'FILE_PROJECT_CONF': '', 'MAXWAITINGJOBS': '3', 'MEMBERS': 'fc0 fc1 fc2 fc3 fc4',
                              'NUMCHUNKS': '5', 'PROJECT_BRANCH': '', 'PROJECT_COMMIT': '', 'PROJECT_DESTINATION': '',
                              'PROJECT_ORIGIN': '', 'PROJECT_PATH': '', 'PROJECT_TYPE': 'none', 'PROJECT_URL': '',
                              'RERUN': 'FALSE', 'SAFETYSLEEPTIME': '10'})

    def test_load_project_parameters(self):
        self.assertDictEqual(self.config.load_project_parameters(),
                             {})

    def test_print_parameters(self):
        self.config.print_parameters('title', self.config.load_parameters())

    def test_get_expid(self):
        self.assertEqual(self.config.get_expid(), 'a000')

    def test_set_expid(self):
        self.config.set_expid('a000')

    def test_get_project_type(self):
        self.assertEqual(self.config.get_project_type(), 'none')

    def test_get_file_project_conf(self):
        self.assertEqual(self.config.get_file_project_conf(), '')

    def test_get_file_jobs_conf(self):
        self.assertEqual(self.config.get_file_jobs_conf(), '')

    def test_get_git_project_origin(self):
        self.assertEqual(self.config.get_git_project_origin(), '')

    def test_get_git_project_branch(self):
        self.assertEqual(self.config.get_git_project_branch(), '')

    def test_get_git_project_commit(self):
        self.assertEqual(self.config.get_git_project_commit(), '')

    def test_get_project_destination(self):
        self.assertEqual(self.config.get_git_project_commit(), '')

    def test_set_git_project_commit(self):
        self.assertEqual(self.config.get_git_project_commit(), '')

    def test_get_svn_project_url(self):
        self.assertEqual(self.config.get_svn_project_url(), '')

    def test_get_svn_project_revision(self):
        self.assertEqual(self.config.get_svn_project_revision(), '')

    def test_get_local_project_path(self):
        self.assertEqual(self.config.get_local_project_path(), '')

    def test_get_date_list(self):
        self.assertEqual(self.config.get_date_list(), [datetime.datetime(1956, 1, 1, 0, 0)])

    def test_get_num_chunks(self):
        self.assertEqual(self.config.get_num_chunks(), 5)

    def test_get_chunk_size_unit(self):
        self.assertEqual(self.config.get_chunk_size_unit(), 'month')

    def test_get_member_list(self):
        self.assertEqual(self.config.get_member_list(), ['fc0', 'fc1', 'fc2', 'fc3', 'fc4'])

    def test_get_rerun(self):
        self.assertEqual(self.config.get_rerun(), 'false')

    def test_get_chunk_list(self):
        self.assertEqual(self.config.get_chunk_list(), '')

    def test_get_platform(self):
        self.assertEqual(self.config.get_platform(), 'patata')

    def test_set_platform(self):
        self.config.set_platform('patata')

    def test_set_version(self):
        self.config.set_version('3.0.0rc1')

    def test_get_total_jobs(self):
        self.assertEqual(self.config.get_total_jobs(), 6)

    def test_get_max_waiting_jobs(self):
        self.assertEqual(self.config.get_max_waiting_jobs(), 3)

    def test_get_safetysleeptime(self):
        self.assertEqual(self.config.get_safetysleeptime(), 10)

    def test_set_safetysleeptime(self):
        self.config.set_safetysleeptime(10)

    def test_get_retrials(self):
        self.assertEqual(self.config.get_retrials(), 4)

    def test_get_parser(self):
        self.assertIsNotNone(self.config.get_parser(self.config._exp_parser_file))

    def test_read_platforms_conf(self):
        self.assertEqual(self.config.get_local_project_path(), '')

    def test_get_option(self):
        self.assertEqual(AutosubmitConfig.get_option(self.config._exp_parser, 'git', 'PROJECT_ORIGIN', ''), '')
        self.assertEqual(AutosubmitConfig.get_option(self.config._exp_parser, 'git', 'PROJECT_ORIGIN', 'origin'),
                         '')
        self.assertEqual(AutosubmitConfig.get_option(self.config._exp_parser, 'DEFAULT', 'HPCARCH', ''),
                         'patata')
        self.assertEqual(AutosubmitConfig.get_option(self.config.jobs_parser, 'INI', 'PROCESSORS', 1),
                         1)
        self.assertEqual(AutosubmitConfig.get_option(self.config.jobs_parser, 'INI', 'PROCESSORS', 2),
                         2)

    def test_get_bool_option(self):
        self.assertFalse(AutosubmitConfig.get_bool_option(self.config._exp_parser, 'rerun', 'RERUN', True))

    def test_check_exists(self):
        self.assertFalse(AutosubmitConfig.check_exists(self.config.jobs_parser, 'INI', 'PROCESSORS'))
        self.assertTrue(AutosubmitConfig.check_exists(self.config._exp_parser, 'DEFAULT', 'HPCARCH'))

    def test_check_is_boolean(self):
        self.assertTrue(AutosubmitConfig.check_is_boolean(self.config._exp_parser, 'rerun', 'RERUN', True))
        self.assertFalse(AutosubmitConfig.check_is_boolean(self.config._exp_parser, 'DEFAULT', 'HPCARCH', True))
        self.assertFalse(AutosubmitConfig.check_is_boolean(self.config.jobs_parser, 'INI', 'PROCESSORS', True))
        self.assertTrue(AutosubmitConfig.check_is_boolean(self.config.jobs_parser, 'INI', 'PROCESSORS', False))

    def test_check_is_choice(self):
        self.assertTrue(AutosubmitConfig.check_is_choice(self.config._exp_parser, 'DEFAULT', 'HPCARCH', True,
                                                         ['patata']))
        self.assertTrue(AutosubmitConfig.check_is_choice(self.config.jobs_parser, 'INI', 'PROCESSORS', False,
                                                         ['patata']))
        self.assertFalse(AutosubmitConfig.check_is_choice(self.config.jobs_parser, 'INI', 'PROCESSORS', True,
                                                          ['patata']))

    def test_check_is_int(self):
        self.assertFalse(AutosubmitConfig.check_is_int(self.config.jobs_parser, 'INI', 'PROCESSORS', True))
        self.assertTrue(AutosubmitConfig.check_is_int(self.config.jobs_parser, 'SIM', 'PROCESSORS', True))

    # def test_check_regex(self):
    #     self.fail()
    #
    # def test_check_json(self):
    #     self.fail()



from unittest import TestCase
from autosubmit.date.chunk_date_lib import *
from datetime import datetime


class TestChunkDateLib(TestCase):
    def test_add_time(self):
        self.assertEqual(add_time(datetime(2000, 1, 1), 1, 'month', 'standard'), datetime(2000, 2, 1))
        self.assertEqual(add_time(datetime(2000, 1, 1), 1, 'day', 'standard'), datetime(2000, 1, 2))
        self.assertEqual(add_time(datetime(2000, 1, 1), 1, 'hour', 'standard'), datetime(2000, 1, 1, 1))

        # Testing noleap calendar
        self.assertEqual(add_time(datetime(2000, 2, 28), 1, 'day', 'noleap'), datetime(2000, 3, 1))
        self.assertEqual(add_time(datetime(2000, 2, 28), 1, 'day', 'standard'), datetime(2000, 2, 29))

    def test_add_years(self):
        self.assertEqual(add_years(datetime(2000, 1, 1), 1), datetime(2001, 1, 1))

    def test_add_months(self):
        self.assertEqual(add_months(datetime(2000, 1, 1), 1, 'standard'), datetime(2000, 2, 1))
        self.assertEqual(add_months(datetime(2000, 1, 29), 1, 'standard'), datetime(2000, 2, 29))
        self.assertEqual(add_months(datetime(2000, 1, 29), 1, 'noleap'), datetime(2000, 2, 28))

    def test_add_days(self):
        self.assertEqual(add_days(datetime(2000, 1, 1), 1, 'standard'), datetime(2000, 1, 2))
        self.assertEqual(add_days(datetime(2000, 2, 28), 1, 'standard'), datetime(2000, 2, 29))
        self.assertEqual(add_days(datetime(2000, 2, 28), 1, 'noleap'), datetime(2000, 3, 1))

    def test_add_hours(self):
        self.assertEqual(add_hours(datetime(2000, 1, 1), 24, 'standard'), datetime(2000, 1, 2))
        self.assertEqual(add_hours(datetime(2000, 1, 1, 23), 1, 'standard'), datetime(2000, 1, 2))
        self.assertEqual(add_hours(datetime(2000, 2, 28), 24, 'standard'), datetime(2000, 2, 29))
        self.assertEqual(add_hours(datetime(2000, 2, 28), 24, 'noleap'), datetime(2000, 3, 1))

    def test_subs_days(self):
        self.assertEqual(sub_days(datetime(2000, 1, 2), 1, 'standard'), datetime(2000, 1, 1))
        self.assertEqual(sub_days(datetime(2000, 3, 1), 1, 'standard'), datetime(2000, 2, 29))
        self.assertEqual(sub_days(datetime(2000, 3, 1), 1, 'noleap'), datetime(2000, 2, 28))

    def test_subs_dates(self):
        self.assertEqual(subs_dates(datetime(2000, 1, 1), datetime(2000, 1, 2), 'standard'), 1)
        self.assertEqual(subs_dates(datetime(2000, 1, 2), datetime(2000, 1, 1), 'standard'), -1)
        self.assertEqual(subs_dates(datetime(2000, 2, 28), datetime(2000, 3, 1), 'standard'), 2)
        self.assertEqual(subs_dates(datetime(2000, 2, 28), datetime(2000, 3, 1), 'noleap'), 1)

    def test_chunk_start_date(self):
        self.assertEqual(chunk_start_date(datetime(2000, 1, 1), 1, 1, 'month', 'standard'),
                         datetime(2000, 1, 1))
        self.assertEqual(chunk_start_date(datetime(2000, 1, 1), 3, 1, 'month', 'standard'),
                         datetime(2000, 3, 1))
        self.assertEqual(chunk_start_date(datetime(2000, 1, 1), 1, 3, 'month', 'standard'),
                         datetime(2000, 1, 1))
        self.assertEqual(chunk_start_date(datetime(2000, 1, 1), 3, 3, 'month', 'standard'),
                         datetime(2000, 7, 1))

        self.assertEqual(chunk_start_date(datetime(2000, 1, 1), 1, 1, 'day', 'standard'),
                         datetime(2000, 1, 1))
        self.assertEqual(chunk_start_date(datetime(2000, 1, 1), 3, 1, 'day', 'standard'),
                         datetime(2000, 1, 3))
        self.assertEqual(chunk_start_date(datetime(2000, 1, 1), 1, 3, 'day', 'standard'),
                         datetime(2000, 1, 1))
        self.assertEqual(chunk_start_date(datetime(2000, 1, 1), 3, 3, 'day', 'standard'),
                         datetime(2000, 1, 7))

        self.assertEqual(chunk_start_date(datetime(2000, 1, 1), 1, 1, 'hour', 'standard'),
                         datetime(2000, 1, 1))
        self.assertEqual(chunk_start_date(datetime(2000, 1, 1), 3, 1, 'hour', 'standard'),
                         datetime(2000, 1, 1, 2))
        self.assertEqual(chunk_start_date(datetime(2000, 1, 1), 1, 3, 'hour', 'standard'),
                         datetime(2000, 1, 1))
        self.assertEqual(chunk_start_date(datetime(2000, 1, 1), 3, 3, 'hour', 'standard'),
                         datetime(2000, 1, 1, 6))

    def test_chunk_end_date(self):
        self.assertEqual(chunk_end_date(datetime(2000, 1, 1), 1, 'month', 'standard'),
                         datetime(2000, 2, 1))
        self.assertEqual(chunk_end_date(datetime(2000, 1, 1), 3, 'month', 'standard'),
                         datetime(2000, 4, 1))

        self.assertEqual(chunk_end_date(datetime(2000, 1, 1), 1, 'day', 'standard'),
                         datetime(2000, 1, 2))
        self.assertEqual(chunk_end_date(datetime(2000, 1, 1), 3, 'day', 'standard'),
                         datetime(2000, 1, 4))

        self.assertEqual(chunk_end_date(datetime(2000, 1, 1), 1, 'hour', 'standard'),
                         datetime(2000, 1, 1, 1))
        self.assertEqual(chunk_end_date(datetime(2000, 1, 1), 3, 'hour', 'standard'),
                         datetime(2000, 1, 1, 3))

    def test_previous_date(self):
        self.assertEqual(previous_day(datetime(2000, 1, 2), 'standard'), datetime(2000, 1, 1))
        self.assertEqual(previous_day(datetime(2000, 3, 1), 'standard'), datetime(2000, 2, 29))
        self.assertEqual(previous_day(datetime(2000, 3, 1), 'noleap'), datetime(2000, 2, 28))

    def test_parse_date(self):
        self.assertEqual(parse_date(''), None)
        self.assertEqual(parse_date('2000'), datetime(2000, 1, 1))
        self.assertEqual(parse_date('200001'), datetime(2000, 1, 1))
        self.assertEqual(parse_date('20000101'), datetime(2000, 1, 1))
        self.assertEqual(parse_date('2000010100'), datetime(2000, 1, 1))
        self.assertEqual(parse_date('200001010000'), datetime(2000, 1, 1))

        with self.assertRaises(ValueError):
            parse_date('200014')
        with self.assertRaises(ValueError):
            parse_date('20000230')
        with self.assertRaises(ValueError):
            parse_date('200')

    def test_date2str(self):
        # noinspection PyTypeChecker
        self.assertEqual(date2str(None), '')
        self.assertEqual(date2str(datetime(2000, 1, 1)), '20000101')
        self.assertEqual(date2str(datetime(2000, 1, 1), 'H'), '2000010100')
        self.assertEqual(date2str(datetime(2000, 1, 1), 'M'), '200001010000')



import textwrap
import os
import shutil
import tempfile
from unittest import TestCase
from autosubmit.platforms.lsfplatform import LsfPlatform


class PlatformsTests(TestCase):
    def setUp(self):
        # have to create the cmd file in the tmp folder before testing
        self.platform = LsfPlatform('a000')
        self.platform.set_host("mn-bsc32")
        self.platform.set_scratch("/gpfs/scratch")
        self.platform.set_project("bsc32")
        self.platform.set_user("bsc32704")
        self.platform.update_cmds()
        self.tmpdir = tempfile.mkdtemp()
        self.scriptname = 'test.cmd'
        self.scriptpath = os.path.join(self.tmpdir, self.scriptname)

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def write_script(self):
        script_content = textwrap.dedent("""\
                    #!/bin/sh
                    ###############################################################################
                    #                   TEST
                    ###############################################################################
                    #
                    #BSUB -J test
                    #BSUB -oo /gpfs/scratch/bsc32/bsc32704/a000/LOG_a000/test_%J.out
                    #BSUB -eo /gpfs/scratch/bsc32/bsc32704/a000/LOG_a000/test_%J.err
                    #BSUB -W 00:01
                    #BSUB -n 1
                    #
                    ###############################################################################
                    echo "test"
                    """)
        open(self.scriptpath, 'w').write(script_content)

    def test_connect(self):
        self.assertTrue(self.platform.connect())

    def test_remote_log_dir(self):
        self.platform.connect()
        self.assertTrue(self.platform.check_remote_log_dir())

    def test_send_file(self):
        self.platform.connect()
        self.write_script()
        self.assertTrue(self.platform.send_file(self.scriptpath,
                                                os.path.join(self.platform.get_remote_log_dir(), self.scriptname)))

    def test_submit_job(self):
        self.platform.connect()
        job_id = self.platform.submit_job(self.scriptname)
        self.assertNotEqual(job_id, 0)

    # def test_send_script(self):
    #    self.platform.connect()
    #    self.assertTrue(self.platform.send_script(self.scriptname))



import os
from unittest import TestCase

import shutil

from autosubmit.config.log import LogFormatter, Log
from logging import LogRecord
import tempfile


class TestLogFormatter(TestCase):

    def setUp(self):
        self.file = LogFormatter(True)
        self.console = LogFormatter(False)

    def test_format(self):
        record = LogRecord('record', Log.DEBUG, 'path', 1, 'Message', ['args'], None)
        self.assertEqual(self.file.format(record), record.asctime + ' Message',)
        self.assertEqual('Message', self.console.format(record))

        record = LogRecord('record', Log.INFO, 'path', 1, 'Message', ['args'], None)
        self.assertEqual(self.file.format(record), record.asctime + ' Message',)
        self.assertEqual('Message', self.console.format(record))

        record = LogRecord('record', Log.RESULT, 'path', 1, 'Message', ['args'], None)
        self.assertEqual(self.file.format(record), record.asctime + ' Message',)
        self.assertEqual(LogFormatter.RESULT + 'Message' + LogFormatter.DEFAULT, self.console.format(record))

        record = LogRecord('record', Log.USER_WARNING, 'path', 1, 'Message', ['args'], None)
        self.assertEqual(self.file.format(record), record.asctime + ' Message',)
        self.assertEqual(LogFormatter.WARNING + 'Message' + LogFormatter.DEFAULT, self.console.format(record))

        record = LogRecord('record', Log.WARNING, 'path', 1, 'Message', ['args'], None)
        self.assertEqual(self.file.format(record), '[WARNING] ' + record.asctime + ' Message',)
        self.assertEqual(LogFormatter.WARNING + '[WARNING] Message' + LogFormatter.DEFAULT,
                         self.console.format(record))

        record = LogRecord('record', Log.ERROR, 'path', 1, 'Message', ['args'], None)
        self.assertEqual(self.file.format(record), '[ERROR] ' + record.asctime + ' Message',)
        self.assertEqual(LogFormatter.ERROR + '[ERROR] Message' + LogFormatter.DEFAULT,
                         self.console.format(record))

        record = LogRecord('record', Log.CRITICAL, 'path', 1, 'Message', ['args'], None)
        self.assertEqual(self.file.format(record), '[CRITICAL] ' + record.asctime + ' Message',)
        self.assertEqual(LogFormatter.ERROR + '[CRITICAL] Message' + LogFormatter.DEFAULT,
                         self.console.format(record))


class TestLog(TestCase):

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    def test_set_file(self):
        Log.set_file(os.path.join(self.tempdir, 'test.log'))

    def test_set_console_level(self):
        Log.set_console_level(Log.NO_LOG)
        Log.set_console_level(Log.EVERYTHING)

    def test_set_file_level(self):
        Log.set_file_level(Log.NO_LOG)
        Log.set_file_level(Log.EVERYTHING)

    def test_debug(self):
        Log.debug('Message')
        Log.debug('{0} {1}', 'complex', 'message')
        with self.assertRaises(IndexError):
            Log.debug('{0} {1}', 'Parameter mising')

    def test_info(self):
        Log.info('Message')
        Log.info('{0} {1}', 'complex', 'message')
        with self.assertRaises(IndexError):
            Log.info('{0} {1}', 'Parameter mising')

    def test_result(self):
        Log.result('Message')
        Log.result('{0} {1}', 'complex', 'message')
        with self.assertRaises(IndexError):
            Log.result('{0} {1}', 'Parameter mising')

    def test_user_warning(self):
        Log.user_warning('Message')
        Log.user_warning('{0} {1}', 'complex', 'message')
        with self.assertRaises(IndexError):
            Log.user_warning('{0} {1}', 'Parameter mising')

    def test_warning(self):
        Log.warning('Message')
        Log.warning('{0} {1}', 'complex', 'message')
        with self.assertRaises(IndexError):
            Log.warning('{0} {1}', 'Parameter mising')

    def test_error(self):
        Log.error('Message')
        Log.error('{0} {1}', 'complex', 'message')
        with self.assertRaises(IndexError):
            Log.error('{0} {1}', 'Parameter mising')

    def test_critical(self):
        Log.critical('Message')
        Log.critical('{0} {1}', 'complex', 'message')
        with self.assertRaises(IndexError):
            Log.critical('{0} {1}', 'Parameter mising')



#!/usr/bin/env python

# Copyright 2015 Earth Sciences Department, BSC-CNS

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
import textwrap


class Status:
    """
    Class to handle the status of a job
    """
    WAITING = 0
    READY = 1
    SUBMITTED = 2
    QUEUING = 3
    RUNNING = 4
    COMPLETED = 5
    FAILED = -1
    UNKNOWN = -2
    SUSPENDED = -3

    def retval(self, value):
        return getattr(self, value)


# noinspection PyPep8
class StatisticsSnippet:
    """
    Class to handle the statistics snippet of a job. It contains header and tailer for
    local and remote jobs
    """

    AS_HEADER = textwrap.dedent("""\

            ###################
            # Autosubmit header
            ###################

            job_name_ptrn='%CURRENT_LOGDIR%/%JOBNAME%'
            echo $(date +%s) > ${job_name_ptrn}_STAT

            ###################
            # Autosubmit job
            ###################

            """)

    # noinspection PyPep8
    AS_TAILER = textwrap.dedent("""\

            ###################
            # Autosubmit tailer
            ###################

            echo $(date +%s) >> ${job_name_ptrn}_STAT
            touch ${job_name_ptrn}_COMPLETED
            exit 0
            """)

